import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter_keyboard_visibility/flutter_keyboard_visibility.dart';
import 'package:get/get.dart';
import 'package:getx_extended/export.dart';

class AppRoot extends StatefulWidget {
  Widget child;
  Widget Function(bool isKeyboardVisible) keyboardToolbar;
  Widget loading;
  Future Function(dynamic error) consumeError;

  AppRoot({
    required this.child,
    required this.keyboardToolbar,
    required this.loading,
    required this.consumeError,
  });

  @override
  _AppRootState createState() => _AppRootState();
}

class _AppRootState extends State<AppRoot> with AutoDisposableState {
  late TaskController _task = Get.find();

  @override
  void initState() {
    super.initState();
    autoDispose(_task.errors.listen((_) => _checkError()));
  }

  void _checkError() {
    final error = _task.errors.firstOrNull;
    if (error == null) return;
    postFrameFuture(() async {
      await widget.consumeError(error);
      _task.dismiss(error);
    });
  }

  @override
  Widget build(BuildContext context) => Scaffold(
        body: KeyboardVisibilityBuilder(
          builder: (_, isKeyboardVisible) => SafeArea(
            top: false,
            bottom: isKeyboardVisible,
            child: Column(
              children: [
                Expanded(
                  child: Stack(
                    children: [
                      widget.child,
                      _task.isProcessing.toWidget(
                        (rx) => rx.value ? widget.loading : Container(),
                      )
                    ],
                  ),
                ),
                widget.keyboardToolbar(isKeyboardVisible)
              ],
            ),
          ),
        ),
      );
}
